@@ -13,16 +13,28 @@
 #
 #      You should have received a copy of the GNU General Public License
 #      along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 import datetime
 import errno
 import json
 import logging
 import os
 from datetime import datetime, timezone
+from json import JSONDecodeError
 from typing import Any, Dict, Final, List, Tuple
 
-from load_shedding.providers import Provider, Stage, ProviderError, StageError, Province, Suburb
-from load_shedding.providers.eskom import Eskom
+from load_shedding.providers import (
+    Area, Provider, ProviderError, Stage, StageError, datetime_to_isoformat, isoformat_to_datetime
+)
+
+logging.basicConfig(
+    level=logging.DEBUG,
+    format="%(asctime)s [%(levelname)s] %(message)s",
+    handlers=[
+        logging.StreamHandler()
+    ]
+)
+logger = logging.getLogger(__name__)
 
 CACHE_TTL: Final = 86400  # 24 hours
 
@@ -31,25 +43,29 @@ class LoadSheddingError(Exception):
     pass
 
 
+def get_areas(provider: Provider, search_text: str = None, max_results: int = None) -> List[Any]:
+    try:
+        areas = provider.get_areas(search_text, max_results)
+    except ProviderError as e:
+        raise ProviderError(f"Unable to get areas from {provider.name}.") from e
+    else:
+        return areas
+
+
 def get_stage(provider: Provider) -> Stage:
     try:
         stage = provider.get_stage()
     except Exception as e:
-        msg = f"Unable to get stage from {provider.name}."
-        logging.error(msg, exc_info=True)
-        raise ProviderError(msg)
+        raise ProviderError(f"Unable to get stage from {provider.name}.") from e
     return stage
 
 
-def get_schedule(provider: Provider, province: Province, suburb: Suburb, stage: Stage = None,
-                 cached: bool = True) -> List[Tuple]:
+def get_schedule(provider: Provider, area: Area, stage: Stage = None, cached: bool = True) -> List[Tuple]:
     try:
         os.makedirs('.cache')
     except OSError as e:
         if e.errno != errno.EEXIST:
-            msg = f"Unable to make cache dir."
-            logging.error(msg, exc_info=True)
-            raise LoadSheddingError(msg)
+            raise LoadSheddingError(f"Unable to make cache dir.") from e
 
     if stage in [None]:
         stage = get_stage(provider)
@@ -60,63 +76,47 @@ def get_schedule(provider: Provider, province: Province, suburb: Suburb, stage:
     cached_schedules = {}
     cached_schedule = []
     now = datetime.now(timezone.utc)
+    cache_file = f".cache/{area.province.value}_{area.id}.json"
     try:
-        cache_file = ".cache/{suburb_id}.json".format(suburb_id=suburb.id)
         with open(cache_file, "r") as cache:
             for k, v in json.loads(cache.read()).items():
                 cached_schedules[Stage(int(k)).value] = v
 
             cached_schedule = cached_schedules.get(stage.value, {})
 
-        if not isinstance(cached_schedule, dict) or len(cached_schedule) == 0:
-            """ Upgrade from previous list """
-            cached = False
-
         if cached and cached_schedule:
             updated_at = datetime.fromisoformat(cached_schedule.get("updated_at", None))
 
             cache_age = (now - updated_at)
             if cache_age.total_seconds() < CACHE_TTL:
-                return cached_schedule.get("schedule", {})
-    except FileNotFoundError as e:
-        logging.error("Unable to get schedule from cache. {e}".format(e=e))
+                schedule = cached_schedule.get("schedule", {})
+                schedule = isoformat_to_datetime(schedule)
+                return schedule
+    except (AttributeError, FileNotFoundError, IndexError, JSONDecodeError, ValueError) as e:
+        logging.debug(f"Unable to get schedule from cache. %s", e, exc_info=True)
 
     try:
-        schedule = provider.get_schedule(province=province, suburb=suburb, stage=stage)
+        schedule = provider.get_schedule(area=area, stage=stage)
     except Exception as e:
         if cached_schedule:
             return cached_schedule  # best effort
-        msg = f"Unable to get schedule from {provider.name}."
-        logging.error(msg, exc_info=True)
-        raise ProviderError(msg)
+        raise ProviderError(f"Unable to get schedule from {provider.name}.") from e
     else:
-        cache_file = ".cache/{suburb_id}.json".format(suburb_id=suburb.id)
         with open(cache_file, "w") as cache:
             cached_schedules[stage.value] = {
                 "updated_at": str(now),
-                "schedule": schedule,
+                "schedule": datetime_to_isoformat(schedule),
             }
             cache.write(json.dumps(cached_schedules))
         return schedule
 
 
-def search(provider: Provider, search_text: str, max_results: int = None) -> List[Any]:
-    try:
-        results = provider.search(search_text, max_results)
-    except Exception as e:
-        msg = f"Unable to search {search_text} from {provider.name}."
-        logging.error(msg, exc_info=True)
-        raise ProviderError(msg)
-    else:
-        return results
-
-
 def list_to_dict(schedule: list) -> Dict:
     schedule_dict = {}
     now = datetime.now(timezone.utc)
     for item in schedule:
-        start = datetime.fromisoformat(item[0])
-        end = datetime.fromisoformat(item[1])
+        start = item[0]
+        end = item[1]
 
     schedule_dict[start.strftime("%Y-%m-%d")] = (
         start.replace(second=now.second).strftime("%H:%M"),
@@ -125,32 +125,9 @@ def list_to_dict(schedule: list) -> Dict:
     return schedule_dict
 
 
-def get_providers() -> List[Provider]:
-    return [Eskom()]
-
-
-def search_suburb(provider: Provider, suburb: str) -> Dict:
-    try:
-        suburb = provider.search_suburb()
-    except ProviderError as e:
-        msg = f"Unable to search suburb from {provider.name}."
-        logging.error(msg, exc_info=True)
-        raise StageError(msg)
-    return suburb
-
-
 def dict_list_to_obj_list(data: List[Dict], T: Any) -> Any:
     objs: List = []
     for d in data:
         s = T(**d)
         objs.append(s)
     return objs
-
-
-def filter_empty_suburbs(suburbs: List[Suburb]) -> List[Suburb]:
-    filtered: List[Suburb] = []
-    for s in suburbs:
-        if not s.total:
-            continue
-        filtered.append(s)
-    return filtered
